LiteX SoC in a Verilated Simulation
============================================

This board is targeting a
[verilated](https://www.veripool.org/wiki/verilator) SoC bitstream
built using [LiteX](https://github.com/enjoy-digital/litex).

Since LiteX is a SoC builder, the individual generated bitstreams can
differ significantly depending on the release and configuration
options used. This board definition currently targets and has been
tested with
- [the LiteX SoC generator, revision
  e0d5a7bff5](https://github.com/enjoy-digital/litex/tree/e0d5a7bff55923)
- using the included
  [lxsim](https://github.com/enjoy-digital/litex/blob/e0d5a7bff55923/litex/tools/litex_sim.py)
- built around a VexRiscv-CPU
- featuring a TIMER0 with 64-bit wide hardware uptime
- along with the following configuration options:

  ```
  --csr-data-width=32
  --integrated-rom-size=0x100000
  --cpu-variant=secure
  --with-ethernet
  --timer-uptime
  --rom-init $PATH_TO_TOCK_BINARY
  ```

Many bitstream customizations can be represented in the Tock board by
simply changing the variables in
`src/litex_generated_constants.rs`. To support a different set of FPGA
cores and perform further modifications, the `src/main.rs` file will
have to be modified.

This board makes assumptions about the generated LiteX SoC, such as
CSR locations in memory. The companion repository
[tock-litex](https://github.com/lschuermann/tock-litex) provides
access to an environment with the required LiteX Python packages in
their targeted versions. This board currently targets the release
[2021081101](https://github.com/lschuermann/tock-litex/releases/tag/2021081101)
of `tock-litex`.


Building the SoC / Running the simulation
-----------------------------------------

Please refer to the [LiteX
documentation](https://github.com/enjoy-digital/litex/wiki/) for
instructions on how to install and use the LiteX SoC generator.

Once LiteX is installed, running the simulation:

```
$ cd $PATH_TO_LITEX/litex/tools
$ ./litex_sim.py <configuration options above>
```

This command will build the SoC Verilog files and a LiteX ROM BIOS.
Afterwards, it will use Verilator to create a C++ simulation program,
which is compiled and run. The script may ask for an administrator
password to create a `tap0` network device on the host if built with
Ethernet support. The Tock kernel is included as the integrated ROM
(loaded at address `0x0`) by using the `--rom-init
$PATH_TO_TOCK_BINARY`.

If everything works you should be greeted by the Tock kernel:
```
...
[xgmii_ethernet] loaded (0xa51090)
[clocker] loaded
[clocker] sys_clk: freq_hz=1000000, phase_deg=0
Verilated LiteX+VexRiscv: initialization complete, entering main loop.
```

Debugging
---------

LiteX makes it easy to generate vastly different SoCs. Prior to
creating an issue on GitHub, please verify that all variables in
`src/litex_generated.rs` correspond to the values provided to LiteX or
generated by it. The respective file paths for the source of each
value is included as a comment.

It is possible to extend the VexRiscv-CPU with a debug port, which is
exposed on the Wishbone bus of the SoC. This can then be used to
attach a GDB debugger via the Network using the Etherbone
(`--with-etherbone`) core. For more information refer to the LiteX
documentation and [this quickstart
guide](https://github.com/timvideos/litex-buildenv/wiki/Debugging).
