#![no_std]

use cortexm4::{generic_isr, unhandled_interrupt};

pub use stm32f4xx::{adc, chip, dbg, dma1, exti, gpio, nvic, rcc, spi, syscfg, tim2, usart};

pub mod interrupt_service;

// Extracted from RM0368 Reference manual, Table 38
#[cfg_attr(all(target_arch = "arm", target_os = "none"), link_section = ".irqs")]
// "used" ensures that the symbol is kept until the final binary
#[cfg_attr(all(target_arch = "arm", target_os = "none"), used)]
pub static IRQS: [unsafe extern "C" fn(); 85] = [
    generic_isr,         // WWDG (0)
    generic_isr,         // PVD (1)
    generic_isr,         // TAMP_STAMP (2)
    generic_isr,         // RTC_WKUP (3)
    generic_isr,         // FLASH (4)
    generic_isr,         // RCC (5)
    generic_isr,         // EXTI0 (6)
    generic_isr,         // EXTI1 (7)
    generic_isr,         // EXTI2 (8)
    generic_isr,         // EXTI3 (9)
    generic_isr,         // EXTI4 (10)
    generic_isr,         // DMA1_Stream0 (11)
    generic_isr,         // DMA1_Stream1 (12)
    generic_isr,         // DMA1_Stream2 (13)
    generic_isr,         // DMA1_Stream3 (14)
    generic_isr,         // DMA1_Stream4 (15)
    generic_isr,         // DMA1_Stream5 (16)
    generic_isr,         // DMA1_Stream6 (17)
    generic_isr,         // ADC (18)
    unhandled_interrupt, // (19)
    unhandled_interrupt, // (20)
    unhandled_interrupt, // (21)
    unhandled_interrupt, // (22)
    generic_isr,         // EXTI9_5 (23)
    generic_isr,         // TIM1_BRK_TIM9 (24)
    generic_isr,         // TIM1_UP_TIM10 (25)
    generic_isr,         // TIM1_TRG_COM_TIM11 (26)
    generic_isr,         // TIM1_CC (27)
    generic_isr,         // TIM2 (28)
    generic_isr,         // TIM3 (29)
    generic_isr,         // TIM4 (30)
    generic_isr,         // I2C1_EV (31)
    generic_isr,         // I2C1_ER (32)
    generic_isr,         // I2C2_EV (33)
    generic_isr,         // I2C2_ER (34)
    generic_isr,         // SPI1 (35)
    generic_isr,         // SPI2 (36)
    generic_isr,         // USART1 (37)
    generic_isr,         // USART2 (38)
    generic_isr,         // USART3 (39)
    generic_isr,         // EXTI15_10 (40)
    generic_isr,         // RTC_Alarm (41)
    generic_isr,         // OTG_FS_WKUP (42)
    unhandled_interrupt, // (43)
    unhandled_interrupt, // (44)
    unhandled_interrupt, // (45)
    unhandled_interrupt, // (45)
    generic_isr,         // DMA1_Stream7 (47)
    unhandled_interrupt, // (48)
    generic_isr,         // SDIO (49)
    generic_isr,         // TIM5 (50)
    generic_isr,         // SPI3 (51)
    unhandled_interrupt, // (52)
    unhandled_interrupt, // (53)
    unhandled_interrupt, // (54)
    unhandled_interrupt, // (55)
    generic_isr,         // DMA2_Stream0 (56)
    generic_isr,         // DMA2_Stream1 (57)
    generic_isr,         // DMA2_Stream2 (58)
    generic_isr,         // DMA2_Stream3 (59)
    generic_isr,         // DMA2_Stream4 (60)
    unhandled_interrupt, // (61)
    unhandled_interrupt, // (62)
    unhandled_interrupt, // (63)
    unhandled_interrupt, // (64)
    unhandled_interrupt, // (65)
    unhandled_interrupt, // (66)
    generic_isr,         // OTG_FS (67)
    generic_isr,         // DMA2_Stream5 (68)
    generic_isr,         // DMA2_Stream6 (69)
    generic_isr,         // DMA2_Stream7 (70)
    generic_isr,         // USART6 (71)
    generic_isr,         // I2C3_EV (72)
    generic_isr,         // I2C3_ER (73)
    unhandled_interrupt, // (74)
    unhandled_interrupt, // (75)
    unhandled_interrupt, // (76)
    unhandled_interrupt, // (77)
    unhandled_interrupt, // (78)
    unhandled_interrupt, // (79)
    unhandled_interrupt, // (80)
    generic_isr,         // FPU (81)
    unhandled_interrupt, // (82)
    unhandled_interrupt, // (83)
    generic_isr,         // SPI4 (84)
];

pub unsafe fn init() {
    stm32f4xx::init();
}
