---
driver number: 0x30002
---

# UDP

## Overview

The UDP driver allows a process to send and receive UDP packets using the
Tock networking stack. Currently, this driver allows for tx and rx of
UDP packets via 6LoWPAN, which sits on top of the 802.15.4 radio.

This driver can be found in capsules/src/net/udp/driver.rs
driver.rs implements an interface for sending
and receiving UDP messages. It also exposes a list of interace addresses to
the application layer. The primary functionality embedded in the UDP driver
is within the allow(), subscribe(), and command() calls which can be made to
the driver.

## Allow

  * Description allow() is used to setup buffers to read/write from. This function takes in
    an `allow_num` and a slice. These allow\_nums determine which buffer is being
    setup as follows:

  * ### Allow Number: 0

    **Description**: Read Buffer.

    **Argument 1**: Slice into which the received payload should be stored

    **Returns**: Ok(())

  * ### Allow Number: 1

    **Description**: Write Buffer.

    **Argument 1**: Slice containing the UDP payload to be transmitted

    **Returns**: Ok(())

  * ### Allow Number: 2

    **Description**: Tx Config Buffer.

    **Argument 1**: Slice containing config information, namely source/destination
                    addresses and ports.
                    Specifically, the config buffer should be the size of two sock_addr_t
                    structs. The first half of the buffer should contain the
                    source address/port (represented as a sock_addr_t)
                    from which the application expects to send.
                    The second half of the buffer should contain the destination
                    address/port which the application wishes to send the next packet to.

    **Returns**: Ok(())

  * ### Allow Number: 3

    **Description**: RX Config Buffer.

    **Argument 1**: Slice containing the Rx config buffer.
                    Used to contain source/destination addresses
                    and ports for receives (separate from `2` because receives may
                    be waiting for an incoming packet asynchronously). Specifically,
                    the rx config buffer should be the size of two sock_addr_t
                    structs. The first half of the buffer should contain the
                    address/port (represented as a sock_addr_t)
                    on which the application is listening.
                    The second half of the buffer should contain the incoming source
                    address/port which the application wishes to listen for.

    **Returns**: Ok(())

## Subscribe

  * Description: subscribe() is used to setup callbacks for when frames are transmitted or received.
    It takes in a callback and a subscribe number. The subscribe number indicates the
    callback type:

  * ### Subscribe Number: 0

    **Description**: Setup callback for when frame is received. This callback cannot be set unless
                     the app is bound to a local UDP endpoint.

    **Argument 1**: The callback

    **Argument 2**: AppId

    **Returns**: RESERVE if the app is not currently bound to a port, Ok(()) otherwise.

  * ### Subscribe Number: 1

    **Description**: Setup callback for when frame is transmitted.

    **Argument 1**: The callback

    **Argument 2**: AppId

    **Returns**: Ok(())

## Command

  * Description: command() is used to get the interface list or to transmit a payload. The action
    taken by the driver is determined by the passed command\_num:

  * ### Command Number: 0

    **Description**: Driver check.

    **Argument 1**: Unused

    **Argument 2**: Unused

    **Argument 3**: Unused

    **Returns**: Ok(())

  * ### Command Number: 1

    **Description**: Get the interface list

    **Argument 1**: Number of requested interface addresses

    **Argument 2**: Unused

    **Argument 3**: AppId

    **Returns**: SuccessWithValue, where value is the total number of interfaces

  * ### Command Number: 2

    **Description**: Transmit Payload

    **Argument 1**: Unused

    **Argument 2**: Unused

    **Argument 3**: AppId

    **Returns**: BUSY is this process already has a pending tx.
                 Returns INVAL if no valid buffer has been loaded into the write buffer,
                 or if the config buffer is the wrong length, or if the destination and source
                 port/address pairs cannot be parsed.
                 Otherwise, returns the result of do_next_tx_immediate(). Notably, a successful
                 transmit can produce two different success values. If success is returned,
                 this simply means that the packet was queued. In this case, the app still
                 still needs to wait for a callback to check if any errors occurred before
                 the packet was passed to the radio. However, if SuccessWithValue
                 is returned with value 1, this means the the packet was successfully passed
                 the radio without any errors, which tells the userland application that it does
                 not need to wait for a callback to check if any errors occured while the packet
                 was being passed down to the radio. Any successful return value indicates that
                 the app should wait for a send_done() callback before attempting to queue another
                 packet.
                 Currently, only will transmit if the app has bound to the port passed in the tx_cfg
                 buf as the source address. If no port is bound, returns RESERVE, if it tries to
                 send on a port other than the port which is bound, returns INVALID.

                 Notably, the currently transmit implementation allows for starvation - an
                 an app with a lower app id can send constantly and starve an app with a
                 later ID.

  * ### Command Number: 3

    **Description**: Bind to the address and port in rx_cfg.
                     This command should be called after allow() is called on the rx_cfg buffer, and
                     after subscribe() is used to set up the recv callback. If this command is called
                     and the address in rx_cfg is 0::0 : 0, this command will reset the option
                     containing the bound port to None, and set the rx callback to None.

    **Argument 1**: Unused

    **Argument 2**: Unused

    **Argument 3**: AppId

    **Returns**: Returns Ok(()) if that addr/port combo is free,
                 returns INVAL if the address requested is not a local interface, or if the port
                 requested is 0. Returns BUSY if that port is already bound to by another app.

  * ### Command Number: 4

    **Description**: Returns the maximum payload that can be transmitted by apps using this driver.
                     This represents the size of the payload buffer in the kernel. Apps can use
                     this syscall to ensure they do not attempt to send too-large messages.

    **Argument 1**: Unused

    **Argument 2**: Unused

    **Argument 3**: AppId

    **Returns**: Returns Ok(())WithValue, where the value is the maximum tx payload length

